# sb::Data::Object
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Data::Object;
use strict;
use sb::Time ();
use sb::Text ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.13';
# 0.13 [2007/09/09] changed default
# 0.12 [2007/08/04] added init_accessor
# 0.11 [2007/07/16] changed return to make them consistent
# 0.10 [2007/07/09] changed url/mail to make them consistent
# 0.09 [2007/01/06] changed create_with_array to make them faster
# 0.08 [2006/07/04] changed constructors to make them simple
# 0.07 [2006/07/04] added default and create_new
# 0.06 [2006/07/04] got rid of "return( undef )"
# 0.05 [2006/03/17] changed bracket rule
# 0.04 [2006/03/03] added coder to get/set coder for id
# 0.03 [2006/03/01] added has_date and changed initialize
# 0.02 [2006/02/26] added table and changed elements
# 0.01 [2005/10/25] changed create to fix a bug
# 0.00 [2005/10/20] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_TIMEFORMAT (){ '%Year%-%Mon%-%Day%T%Hour%:%Min%:%Sec%%ZoneLong%' }
my $aCoder = 'md5';
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'_erased'   => undef,
		@_
	);
}
# --------------------------------------
# // public functions - constructor
sub create
{
	my $class = shift;
	my %param = @_;
	return if (! defined($param{'id'}));
	return $class->new(%param);
}
sub create_with_array
{
	my $class = shift;
	return $class->create(map { $_ => shift(@_) } $class->elements);
}
sub create_new
{
	my $class = shift;
	my %param = @_;
	my $self = $class->create('id'=>$param{'id'});
	return if (!$self);
	$self->default(%param); # set default
	return $self;
}
sub clone
{
	my $self = shift;
	my $id = shift;
	return $self if ($self->id eq $id);
	my $class = ref($self);
	my %param = map { $_ => $self->{$_} } $self->elements;
	$param{'id'} = $id;
	return $class->create(%param);
}
# --------------------------------------
# // public functions - destructor
sub erase
{ # erase is not actual destructor, but it works as preparation of destruction.
	my $self = shift;
	return if ( !$self->existed or $self->erased );
	foreach my $elem ($self->elements)
	{
		next if ($elem eq 'id');
		$self->{$elem} = undef;
	}
	$self->{'_erased'} = 1;
	return $self;
}
# --------------------------------------
# // public functions - class method
sub table
{
	my $class = shift;
	return ( 'id' => 'uni_id' );
}
sub elements
{
	my $class = shift;
	my $num = 0;
	return grep( ++$num % 2 , $class->table );
}
sub coder
{
	my $class = shift;
	$aCoder = shift if (@_);
	return $aCoder;
}
sub init_accessor
{
	my $class = shift;
	no strict 'refs';
	my @elems = $class->elements;
	my $check = $elems[1]; # $elems[0] is always 'id', so we need to check second one.
	# create accessor functions when the first object is allocated.
	if ($check ne '' and ! defined(&{$class . '::' . $check}))
	{
		my %table = $class->table;
		foreach my $member ( @elems )
		{
			next if ($member eq 'id' or $member eq 'url' or $member eq 'mail'); # reserved
			my $type = $table{$member};
			if (index($type,'date') > -1 and ! defined(&{$class . '::has_date'}))
			{
				*{$class . '::has_date'} = sub { return 1 };
			}
			if (! defined(&{$class . '::' . $member}))
			{
				*{$class . '::' . $member} = sub {
					my $self = shift;
					$self->{$member} = shift if @_;
					$self->{$member};
				};
			} # end of if (! defined(&{$class . '::' . $member}))
		} # end of foreach my $member ( @{$param{'content_list'}} )
	} # end of if ($check ne '' and ! defined(&{$class . '::' . $check}))
}
# --------------------------------------
# // public functions - accessor
sub id
{ # read only
	my $self = shift;
	return $self->{'id'};
}
sub coded_id
{
	my $self = shift;
	return sb::Text->new($self->id)->format($self->coder,'encode');
}
sub url
{
	my $self = shift;
	$self->{'url'} = shift if (@_);
	return if ( !_check_url($self->{'url'}) );
	return $self->{'url'};
}
sub mail
{
	my $self = shift;
	$self->{'mail'} = shift if (@_);
	return if ( !_check_mail($self->{'mail'}) );
	return $self->{'mail'};
}
sub entitize
{ # read only
	my $self = shift;
	my $key  = shift;
	return if (!$key);
	return sb::Text->get($self->{$key})->entitized;
}
sub formed_date
{ # read only
	my $self = shift;
	my %param = (
		'type'   => 'mod', # gen or mod
		'format' => DEFAULT_TIMEFORMAT,
		@_
	);
	my $type = $param{'type'};
	return if (!$self->has_date);
	return if ($type ne 'gen' and $type ne 'mod');
	return sb::Time->format(
		'time' => $self->{$type},
		'form' => $param{'format'},
		'zone' => $self->{'zone'} || '+0000',
	);
}
sub set_variables
{ # write only (set variables except id)
	my $self = shift;
	my %param = @_;
	foreach my $elem ($self->elements)
	{
		next if ($elem eq 'id');
		$self->{$elem} = $param{$elem};
	}
	return;
}
# --------------------------------------
# // public functions - common methods
sub existed
{
	my $self = shift;
	return ($self->{'id'} ne '');
}
sub erased
{
	my $self = shift;
	return $self->{'_erased'};
}
sub has_date
{
	return;
}
sub default
{
	my $self = shift;
	$self->initialize(@_);
}
# --------------------------------------
# // private functions
sub _check_url
{
	return ($_[0] =~ /s?https?:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+/);
}
sub _check_mail
{
	return ($_[0] =~ /[\w=+\$%*-]+\@[^\s()\[\]{}!\"\'<>:,\x7f-\xff]+\.\w+/);
}
1;
__END__
